@@ -23,9 +23,11 @@ module Rake
       @cross_compile = false
       @cross_config_options = []
       @cross_compiling = nil
-      @no_native = false
+      @no_native = (ENV["RAKE_EXTENSION_TASK_NO_NATIVE"] == "true")
       @config_includes = []
-      @ruby_versions_per_platform = {}
+      # Default to an empty list of ruby versions for each platform
+      @ruby_versions_per_platform = Hash.new { |h, k| h[k] = [] }
+      @make = nil
     end
 
     def cross_platform
@@ -45,15 +47,6 @@ module Rake
     end
 
     def define
-      if (defined?(RUBY_ENGINE) && RUBY_ENGINE == 'ironruby')
-        warn_once <<-EOF
-WARNING: You're attempting to (cross-)compile C extensions from a platform
-(#{RUBY_ENGINE}) that does not support native extensions or mkmf.rb.
-Rerun `rake` under MRI Ruby 1.8.x/1.9.x to cross/native compile.
-        EOF
-        return
-      end
-
       super
 
       unless compiled_files.empty?
@@ -89,9 +82,12 @@ Rerun `rake` under MRI Ruby 1.8.x/1.9.x to cross/native compile.
     private
     # copy other gem files to staging directory
     def define_staging_file_tasks(files, lib_path, stage_path, platf, ruby_ver)
+      # lib_binary_path
+      lib_binary_path = "#{lib_path}/#{File.basename(binary(platf))}"
+
       files.each do |gem_file|
         # ignore directories and the binary extension
-        next if File.directory?(gem_file) || gem_file == "#{lib_path}/#{binary(platf)}"
+        next if File.directory?(gem_file) || gem_file == lib_binary_path
         stage_file = "#{stage_path}/#{gem_file}"
 
         # copy each file from base to stage directory
@@ -112,38 +108,57 @@ Rerun `rake` under MRI Ruby 1.8.x/1.9.x to cross/native compile.
       platf = for_platform || platform
 
       binary_path = binary(platf)
+      binary_base_name = File.basename(binary_path)
 
       # lib_path
       lib_path = lib_dir
 
-      lib_binary_path = "#{lib_path}/#{binary_path}"
-      lib_binary_dir_path = File.dirname(lib_binary_path)
+      # lib_binary_path
+      lib_binary_path = "#{lib_path}/#{binary_base_name}"
 
       # tmp_path
       tmp_path = "#{@tmp_dir}/#{platf}/#{@name}/#{ruby_ver}"
       stage_path = "#{@tmp_dir}/#{platf}/stage"
 
+      siteconf_path = "#{tmp_path}/.rake-compiler-siteconf.rb"
       tmp_binary_path = "#{tmp_path}/#{binary_path}"
       tmp_binary_dir_path = File.dirname(tmp_binary_path)
-      stage_binary_path = "#{stage_path}/#{lib_path}/#{binary_path}"
+      stage_binary_path = "#{stage_path}/#{lib_binary_path}"
       stage_binary_dir_path = File.dirname(stage_binary_path)
 
       # cleanup and clobbering
       CLEAN.include(tmp_path)
       CLEAN.include(stage_path)
-      CLOBBER.include("#{lib_path}/#{binary(platf)}")
+      CLOBBER.include(lib_binary_path)
       CLOBBER.include("#{@tmp_dir}")
 
       # directories we need
       directory tmp_path
       directory tmp_binary_dir_path
-      directory lib_binary_dir_path
+      directory lib_path
       directory stage_binary_dir_path
 
+      directory File.dirname(siteconf_path)
+      # Set paths for "make install" destinations
+      file siteconf_path => File.dirname(siteconf_path) do
+        File.open(siteconf_path, "w") do |siteconf|
+          siteconf.puts "require 'rbconfig'"
+          siteconf.puts "require 'mkmf'"
+          siteconf.puts "dest_path = mkintpath(#{File.expand_path(lib_path).dump})"
+          %w[sitearchdir sitelibdir].each do |dir|
+            siteconf.puts "RbConfig::MAKEFILE_CONFIG['#{dir}'] = dest_path"
+            siteconf.puts "RbConfig::CONFIG['#{dir}'] = dest_path"
+          end
+        end
+      end
+
       # copy binary from temporary location to final lib
       # tmp/extension_name/extension_name.{so,bundle} => lib/
-      task "copy:#{@name}:#{platf}:#{ruby_ver}" => [lib_binary_dir_path, tmp_binary_path] do
-        install tmp_binary_path, "#{lib_path}/#{binary_path}"
+      task "copy:#{@name}:#{platf}:#{ruby_ver}" => [lib_path, tmp_binary_path, "#{tmp_path}/Makefile"] do
+        # install in lib for native platform only
+        unless for_platform
+          sh "#{make} install target_prefix=", chdir: tmp_path
+        end
       end
       # copy binary from temporary location to staging directory
       task "copy:#{@name}:#{platf}:#{ruby_ver}" => [stage_binary_dir_path, tmp_binary_path] do
@@ -164,20 +179,20 @@ Java extension should be preferred.
 
         chdir tmp_path do
           sh make
-          if binary_path != File.basename(binary_path)
-            cp File.basename(binary_path), binary_path
+          if binary_path != binary_base_name
+            cp binary_base_name, binary_path
           end
         end
       end
 
       # makefile depends of tmp_dir and config_script
       # tmp/extension_name/Makefile
-      file "#{tmp_path}/Makefile" => [tmp_path, extconf] do |t|
+      file "#{tmp_path}/Makefile" => [tmp_path, extconf, siteconf_path] do |t|
         options = @config_options.dup
 
         # include current directory
         include_dirs = ['.'].concat(@config_includes).uniq.join(File::PATH_SEPARATOR)
-        cmd = [Gem.ruby, "-I#{include_dirs}"]
+        cmd = [Gem.ruby, "-I#{include_dirs}", "-r#{File.basename(siteconf_path)}"]
 
         # build a relative path to extconf script
         abs_tmp_path = (Pathname.new(Dir.pwd) + tmp_path).realpath
@@ -226,7 +241,7 @@ Java extension should be preferred.
       # platform matches the indicated one.
       if platf == RUBY_PLATFORM then
         # ensure file is always copied
-        file "#{lib_path}/#{binary_path}" => ["copy:#{name}:#{platf}:#{ruby_ver}"]
+        file lib_binary_path => ["copy:#{name}:#{platf}:#{ruby_ver}"]
 
         task "compile:#{@name}" => ["compile:#{@name}:#{platf}"]
         task "compile" => ["compile:#{platf}"]
@@ -242,6 +257,12 @@ Java extension should be preferred.
       # lib_path
       lib_path = lib_dir
 
+      # lib_binary_path
+      lib_binary_path = "#{lib_path}/#{File.basename(binary(platf))}"
+
+      # Update compiled platform/version combinations
+      @ruby_versions_per_platform[platf] << ruby_ver
+
       # create 'native:gem_name' and chain it to 'native' task
       unless Rake::Task.task_defined?("native:#{@gem_spec.name}:#{platf}")
         task "native:#{@gem_spec.name}:#{platf}" do |t|
@@ -254,7 +275,7 @@ Java extension should be preferred.
           spec.platform = Gem::Platform.new(platf)
 
           # set ruby version constraints
-          ruby_versions = @ruby_versions_per_platform[platf] || []
+          ruby_versions = @ruby_versions_per_platform[platf]
           sorted_ruby_versions = ruby_versions.sort_by do |ruby_version|
             ruby_version.split(".").collect(&:to_i)
           end
@@ -318,13 +339,13 @@ Java extension should be preferred.
       end
 
       # add binaries to the dependency chain
-      task "native:#{@gem_spec.name}:#{platf}" => ["#{stage_path}/#{lib_path}/#{binary(platf)}"]
+      task "native:#{@gem_spec.name}:#{platf}" => ["#{stage_path}/#{lib_binary_path}"]
 
       # ensure the extension get copied
-      unless Rake::Task.task_defined?("#{lib_path}/#{binary(platf)}") then
-        file "#{lib_path}/#{binary(platf)}" => ["copy:#{@name}:#{platf}:#{ruby_ver}"]
+      unless Rake::Task.task_defined?(lib_binary_path) then
+        file lib_binary_path => ["copy:#{@name}:#{platf}:#{ruby_ver}"]
       end
-      file "#{stage_path}/#{lib_dir}/#{binary(platf)}" => ["copy:#{@name}:#{platf}:#{ruby_ver}"]
+      file "#{stage_path}/#{lib_binary_path}" => ["copy:#{@name}:#{platf}:#{ruby_ver}"]
 
       # Allow segmented packaging by platform (open door for 'cross compile')
       task "native:#{platf}" => ["native:#{@gem_spec.name}:#{platf}"]
@@ -356,10 +377,6 @@ Java extension should be preferred.
           @lib_dir = "#{@lib_dir}/#{$1}"
         end
 
-        # Update cross compiled platform/version combinations
-        ruby_versions = (@ruby_versions_per_platform[for_platform] ||= [])
-        ruby_versions << version
-
         define_cross_platform_tasks_with_version(for_platform, version)
 
         # restore lib_dir
@@ -376,6 +393,7 @@ Java extension should be preferred.
         return
       end
 
+      require "yaml"
       config_file = YAML.load_file(config_path)
 
       # tmp_path
@@ -384,6 +402,9 @@ Java extension should be preferred.
       # lib_path
       lib_path = lib_dir
 
+      # lib_binary_path
+      lib_binary_path = "#{lib_path}/#{File.basename(binary(for_platform))}"
+
       unless rbconfig_file = config_file["rbconfig-#{for_platform}-#{ruby_ver}"] then
         warn "no configuration section for specified version of Ruby (rbconfig-#{for_platform}-#{ruby_ver})"
         return
@@ -403,8 +424,21 @@ Java extension should be preferred.
       # genearte fake.rb for different ruby versions
       file "#{tmp_path}/fake.rb" => [rbconfig_file] do |t|
         File.open(t.name, 'w') do |f|
-          f.write fake_rb(for_platform, ruby_ver)
-          f.write File.read(t.prerequisites.first)
+          # Keep the original RbConfig::CONFIG["ENABLE_SHARED"] to use
+          # the same RubyGems extension directory. See also
+          # Gem::BasicSpecificaion#extenions_dir and
+          # Gem.extension_api_version.
+          #
+          # if RbConfig::CONFIG["ENABLE_SHARED"] == "no"
+          #   "extensions/x86_64-linux/2.5.0-static"
+          # else
+          #   "extensions/x86_64-linux/2.5.0"
+          # end
+          f.puts("require 'rbconfig'")
+          f.puts("original_enable_shared = RbConfig::CONFIG['ENABLE_SHARED']")
+          f.puts(fake_rb(for_platform, ruby_ver))
+          f.puts(File.read(t.prerequisites.first))
+          f.puts("RbConfig::CONFIG['ENABLE_SHARED'] = original_enable_shared")
         end
       end
 
@@ -437,12 +471,12 @@ Java extension should be preferred.
 
         # clear lib/binary dependencies and trigger cross platform ones
         # check if lib/binary is defined (damn bundle versus so versus dll)
-        if Rake::Task.task_defined?("#{lib_path}/#{binary(for_platform)}") then
-          Rake::Task["#{lib_path}/#{binary(for_platform)}"].prerequisites.clear
+        if Rake::Task.task_defined?(lib_binary_path) then
+          Rake::Task[lib_binary_path].prerequisites.clear
         end
 
         # FIXME: targeting multiple platforms copies the file twice
-        file "#{lib_path}/#{binary(for_platform)}" => ["copy:#{@name}:#{for_platform}:#{ruby_ver}"]
+        file lib_binary_path => ["copy:#{@name}:#{for_platform}:#{ruby_ver}"]
 
         # if everything for native task is in place
         if @gem_spec && @gem_spec.platform == 'ruby' then
